@@ -23,7 +23,7 @@
 #' write_file(x, tmp)
 #' identical(x, read_file(tmp))
 #'
-#' read_lines(x)
+#' read_lines(I(x))
 read_file <- function(file, locale = default_locale()) {
   if (empty_file(file)) {
     return("")
@@ -54,6 +54,8 @@ write_file <- function(x, file, append = FALSE, path = deprecated()) {
     file <- path
   }
 
+  force(x)
+
   file <- standardise_path(file, input = FALSE)
   if (!isOpen(file)) {
     on.exit(close(file), add = TRUE)
