@@ -3,7 +3,7 @@
 #' This is useful if you need to do some manual munging - you can read the
 #' columns in as character, clean it up with (e.g.) regular expressions and
 #' then let readr take another stab at parsing it. The name is a homage to
-#' the base \code{\link[utils]{type.convert}()}.
+#' the base [utils::type.convert()].
 #'
 #' @param df A data frame.
 #' @param col_types One of `NULL`, a [cols()] specification, or
@@ -12,6 +12,9 @@
 #'   If `NULL`, column types will be imputed using all rows.
 #' @inheritParams tokenizer_delim
 #' @inheritParams read_delim
+#' @note `type_convert()` removes a 'spec' attribute,
+#' because it likely modifies the column data types.
+#' (see [spec()] for more information about column specifications).
 #' @export
 #' @examples
 #' df <- data.frame(
@@ -29,7 +32,8 @@
 #'
 #' # first read the data as character
 #' data <- read_csv(readr_example("mtcars.csv"),
-#'                  col_types = cols(.default = col_character()))
+#'   col_types = list(.default = col_character())
+#' )
 #' str(data)
 #' # Then convert it with type_convert
 #' type_convert(data)
@@ -38,30 +42,34 @@ type_convert <- function(df, col_types = NULL, na = c("", "NA"), trim_ws = TRUE,
   stopifnot(is.data.frame(df))
   is_character <- vapply(df, is.character, logical(1))
 
+  if (!any(is_character)) {
+    warning("`type_convert()` only converts columns of type 'character'.\n- `df` has no columns of type 'character'", call. = FALSE)
+  }
+
   char_cols <- df[is_character]
 
   col_types <- keep_character_col_types(df, col_types)
 
-  guesses <- lapply(char_cols, function(x) {
-    x[x %in% na] <- NA
-    guess_parser(x, locale)
-  })
+  guesses <- lapply(char_cols, guess_parser, locale = locale, na = na)
 
   specs <- col_spec_standardise(
     col_types = col_types,
     col_names = names(char_cols),
-    guessed_types =  guesses
+    guessed_types = guesses
   )
 
-  if (is.null(col_types)) {
+  if (is.null(col_types) && !is_testing()) {
     show_cols_spec(specs)
   }
 
   df[is_character] <- lapply(seq_along(char_cols), function(i) {
     type_convert_col(char_cols[[i]], specs$cols[[i]], which(is_character)[i],
-      locale_ = locale, na = na, trim_ws = trim_ws)
+      locale_ = locale, na = na, trim_ws = trim_ws
+    )
   })
 
+  attr(df, "spec") <- NULL
+
   df
 }
 
@@ -80,7 +88,8 @@ keep_character_col_types <- function(df, col_types) {
         "`df` and `col_types` must have consistent lengths:\n",
         "  * `df` has length ", length(df), "\n",
         "  * `col_types` has length ", nchar(col_types),
-        call. = FALSE)
+        call. = FALSE
+      )
     }
 
     idx <- which(is_character)
