@@ -1,6 +1,4 @@
-context("read_file")
-
-# df <- dplyr::data_frame(français = "élève")
+# df <- dplyr::tibble(français = "élève")
 # write.csv(df,
 #   "tests/testthat/enc-iso-8859-1.txt",
 #   fileEncoding = "ISO-8859-1",
@@ -8,6 +6,8 @@ context("read_file")
 #   quote = FALSE)
 
 test_that("read_file respects encoding", {
+  skip_on_os("solaris")
+
   x <- read_file("enc-iso-8859-1.txt", locale(encoding = "ISO-8859-1"))
   expect_equal(substr(x, 5, 5), "\u00e7")
 })
@@ -22,7 +22,7 @@ test_that("read_file works with a local text file passed as character", {
 test_that("read_file works with a local text file, skipping one line", {
   expect_equal(
     read_file(datasource("sample_text.txt", skip = 1)),
-    paste(tail(strsplit(sample_text_str,"\n")[[1]], -1), collapse = "\n")
+    paste(tail(strsplit(sample_text_str, "\n")[[1]], -1), collapse = "\n")
   )
 })
 
@@ -48,17 +48,21 @@ test_that("read_file works with compressed files", {
 })
 
 test_that("read_file works via https", {
-  url <- "https://raw.githubusercontent.com/hadley/readr/master/tests/testthat/eol-cr.txt"
+  skip_on_cran()
+
+  url <- "https://raw.githubusercontent.com/tidyverse/readr/master/tests/testthat/eol-cr.txt"
   expect_equal(read_file(url), eol_cr_text)
 })
 
 test_that("read_file works via https on gz file", {
-  url <- "https://raw.githubusercontent.com/hadley/readr/master/tests/testthat/eol-cr.txt.gz"
+  skip_on_cran()
+
+  url <- "https://raw.githubusercontent.com/tidyverse/readr/master/tests/testthat/eol-cr.txt.gz"
   expect_equal(read_file(url), eol_cr_text)
 })
 
 test_that("read_file returns \"\" on an empty file", {
-   expect_equal(read_file("empty-file"), "")
+  expect_equal(read_file("empty-file"), "")
 })
 
 # read_file_raw ---------------------------------------------------------------
